.module IO
.using Parent

; ---------------------------------------------------------
; SendHLString -> Send string pointed to by HL.
; ---------------------------------------------------------
; Inputs:    hl -> string to send.
; Outputs:   z = string sent; nz = failed.
; Destroyed: af, hl.
; ---------------------------------------------------------
WriteHLString
	ld a,(hl)
	inc hl
	or a
	ret z
	call TiLP.WriteByte
	ret nz
	jr WriteHLString

; ---------------------------------------------------------
; WriteHLStringCR -> Send string pointed to by HL + '\r'.
; ---------------------------------------------------------
; Inputs:    hl -> string to send.
; Outputs:   z = string sent; nz = failed.
; Destroyed: af, hl.
; ---------------------------------------------------------
WriteHLStringCR
	call WriteHLString
	ret nz
	ld a,'\r'
	jp TiLP.WriteByte

; ---------------------------------------------------------
; WriteOP1 -> Send string contained in OP1..OP6.
; ---------------------------------------------------------
; Inputs:    OP1 = string to send.
; Outputs:   z = string sent; nz = failed.
; Destroyed: af, b, hl.
; ---------------------------------------------------------
WriteOP1
	ld b,66
	ld hl,OP1
-	ld a,(hl)
	inc hl
	or a
	ret z
	call TiLP.WriteByte
	ret nz
	djnz {-}
	xor a
	ret

; ---------------------------------------------------------
; SetShortMode -> Set short command mode.
; ---------------------------------------------------------
; Inputs:    None.
; Outputs:   z = switched to short mode; nz = failed.
; Destroyed: af.
; ---------------------------------------------------------
SetShortMode
	ld a,'\r'
	call TiLP.WriteByte
	ret nz
	call TiLP.ReadByte
	ret nz
	cp '>'
	jr z,ReadToCR ; OK, already in SCS.

	; Read to end of response anyway.
	call ReadToCR
	ret nz
	
	call Delay

	push hl
	ld hl,Commands.ShortMode
	call WriteHLStringCR
	pop hl
	ret nz
	call TiLP.ReadByte
	ret nz
	cp '>'
	jr z,ReadToCR
	call ReadToCR
	xor a
	inc a
	ret

; ---------------------------------------------------------
; ReadToCR -> Read until a '\r' is hit.
; ---------------------------------------------------------
; Inputs:    None.
; Outputs:   z = CR received; nz = failed.
; Destroyed: af.
; ---------------------------------------------------------
ReadToCR
	call TiLP.ReadByte
	ret nz
	cp '\r'
	jr nz,ReadToCR
	jp Delay

; ---------------------------------------------------------
; WriteShortByteCR -> Send a byte followed by '\r'
; ---------------------------------------------------------
; Inputs:    None.
; Outputs:   z = success; nz = failure.
; Destroyed: af.
; Remarks:   Puts the device in short command mode first.
; ---------------------------------------------------------
WriteShortByteCR
	push af
	call SetShortMode
	jr z,{+}
	pop af
	or a
	ret
+
	pop af
	call Delay
	; Run on...

; ---------------------------------------------------------
; WriteByteCR -> Send a byte followed by '\r'
; ---------------------------------------------------------
; Inputs:    None.
; Outputs:   z = success; nz = failure.
; Destroyed: af.
; ---------------------------------------------------------
WriteByteCR
	call TiLP.WriteByte
	ret nz
	ld a,'\r'
	jp TiLP.WriteByte


; ---------------------------------------------------------
; ExecuteAndCheck -> Sends a '\r' and checks the response.
; ---------------------------------------------------------
; Inputs:    None.
; Outputs:   z = success; nz = failure.
; Destroyed: af, b, hl, OP1.
; ---------------------------------------------------------
ExecuteAndCheck
	ld a,'\r'
	call TiLP.WriteByte
	ret nz
	ld hl,0
-	in a,(bport)
	and %11
	cp %11
	jr nz,CheckPromptResponse
	dec hl
	ld a,h
	or l
	jr nz,{-}
	inc a
	ret


; ---------------------------------------------------------
; CheckPromptResponse -> Checks the response.
; ---------------------------------------------------------
; Inputs:    a = command.
; Outputs:   z = success; nz = failure.
; Destroyed: af, b, hl, OP1.
; ---------------------------------------------------------
CheckPromptResponse
	call ReadOP1
	ret nz
	ld a,(OP1)
	cp '>'
	ret nz
	ld a,(OP1+1)
	cp '\r'
	ret nz
	
; ---------------------------------------------------------
; ReadOP1 -> Read a value and store it in the OP1 area.
; ---------------------------------------------------------
; Inputs:    None.
; Outputs:   z = success; nz = failure.
;            hl -> byte after last one received.
; Destroyed: af, b, hl.
; ---------------------------------------------------------
ReadOP1
	ld b,66
	ld hl,OP1
-	call TiLP.ReadByte
	ret nz
	ld (hl),a
	inc hl
	cp '\r'
	ret z
	djnz {-}
	inc b
	ret
	
; ---------------------------------------------------------
; WriteNamedCommandOP1 -> Send a command using OP1 as an
;                         argument.
; ---------------------------------------------------------
; Inputs:    OP1 = named argument.
;            a   = command.
; Outputs:   z = success; nz = failure.
; Destroyed: af, bc, de, hl.
; ---------------------------------------------------------
WriteNamedCommandOP1
	push af
	call SetShortMode
	pop bc
	ret nz
	ld a,b
	call TiLP.WriteByte
	ret nz
	ld a,' '
	call TiLP.WriteByte
	ret nz
	call WriteOP1
	ret nz
	jp ExecuteAndCheck

; ---------------------------------------------------------
; Delay -> Short delay.
; ---------------------------------------------------------
Delay
	ei
	halt
	ret



.endmodule